@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 open Ident
@@ -158,9 +158,8 @@ let compile_let_decl env decl =
   let e,lsize = compile_expr env decl.Typed.let_body in
   let env = enter_globals env (Patterns.fv pat) in
 
-  let comp = 
-    Patterns.Compile.make_branches 
-      (Types.descr (Patterns.accept pat)) [ pat, () ] in
+  let te = decl.Typed.let_body.Typed.exp_typ in 
+  let comp = Patterns.Compile.make_branches te [ pat, () ] in
   let (disp, n) = 
     match comp with
       | (disp, [| Auto_pat.Match (n, ()) |]) -> (disp,n)
@@ -244,7 +243,7 @@ let do_open (tenv,cenv,codes) loc path =
 
 let rec collect_funs accu = function
   | { descr = Ast.FunDecl e } :: rest -> collect_funs (e::accu) rest
-  | rest -> (accu,rest)
+  | rest -> (List.rev accu,rest)
 
 let rec collect_types accu = function
   | { descr = Ast.TypeDecl ((loc,x),t) } :: rest -> 
