@@ -178,10 +178,14 @@ let ocaml_stdlib () =
   Sys.remove "ocaml_stdlib";
   s
 
+let make_absolute dir =
+  if Filename.is_relative dir 
+  then Filename.concat (Sys.getcwd ()) dir
+  else dir
+
 let ml_interface = 
   let dir1 = !(List.assoc "mliface" vars) in
-  let dirs = [] in
-  let dirs = if dir1 = "" then dirs else dir1 :: dirs in
+  let dirs = if dir1 = "" then [] else [ make_absolute dir1 ] in
   print "ocaml sources... ";
   let rec loop = function
     | [] -> 
