@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 open Location
@@ -197,7 +197,8 @@ let debug ppf tenv cenv = function
       Format.fprintf ppf "[DEBUG:sample]@.";
       (try
 	 let t = Types.descr (Typer.typ tenv t) in
-	 Format.fprintf ppf "%a@." print_sample (Sample.get t)
+	 Format.fprintf ppf "%a@." print_sample (Sample.get t);
+	 Format.fprintf ppf "witness: %a@." Types.Witness.print_witness (Types.witness t);
        with Not_found ->
 	 Format.fprintf ppf "Empty type : no sample !@.")
   | `Filter (t,p) -> 
@@ -285,14 +286,16 @@ let show ppf id t v =
     print_norm t 
     print_value_opt v
 
-let phrases ppf phs =
-  let (tenv,cenv,_) = 
-    Compile.comp_unit 
-      ~run:true ~show:(show ppf) 
-      ~directive:(directive ppf)
+let ev_top ~run ~show ?directive phs =
+  let (tenv,cenv,_) =  
+    Compile.comp_unit ~run ~show ?directive
       !typing_env !compile_env phs in
   typing_env := tenv;
   compile_env := cenv
+  
+
+let phrases ppf phs =
+  ev_top ~run:true ~show:(show ppf)  ~directive:(directive ppf) phs
 
 let catch_exn ppf_err exn =
   if not catch_exceptions then raise exn;
@@ -367,8 +370,7 @@ let eval s =
 	  vals := (None,v) :: !vals
       | _ -> assert false
   in
-  ignore (Compile.comp_unit 
-	    ~run:true ~show Builtin.env Compile.empty_toplevel phs);
+  ev_top ~run:true ~show phs;
   List.rev !vals
   
 let eval s =
