@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 open Location
@@ -58,9 +58,6 @@ let mk name descr typing compile code ext_info depends =
 
 let magic = "CDUCE:compunit:00007"
 
-let obj_path = ref [ "" ]
-
-
 let has_obj n =
   let base = U.to_string n ^ ".cdo" in
   List.exists (fun p -> Sys.file_exists (Filename.concat p base)) !obj_path
@@ -134,7 +131,7 @@ let compile_save verbose name src out =
   set_hash c;
   let pools = Value.extract_all () in
 
-  let oc = open_out out in
+  let oc = open_out_bin out in
   output_string oc magic;
   
   Marshal.to_channel oc (pools,c) [];
@@ -158,7 +155,7 @@ let reg_types = ref true
 
 let rec real_load src =
   let ic = 
-    try open_in src
+    try open_in_bin src
     with Sys_error _ -> raise (CannotOpen src) in
   try
     let s = String.copy magic in
@@ -238,7 +235,10 @@ let () =
   Eval.get_builtin := Hashtbl.find static_externals
 
 
-let stub_ml = ref (fun _ _ _ _ _ -> assert false)
+let stub_ml = ref (fun _ _ _ _ _ -> 
+		     Printf.eprintf
+		       "Fatal error: no support for the OCaml interface.\n";
+		     exit 2)
 
 let prepare_stub src =
   let c = real_load src in
