@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 open Ident
@@ -35,8 +35,8 @@ let specs =
              "(for --compile) show types of exported values";
     "--obj-dir",  Arg.String (fun s -> out_dir := s :: !out_dir),
              "(for --compile) directory for the compiled .cdo file";
-    "-I", Arg.String (fun s -> Librarian.obj_path := s::!Librarian.obj_path),
-      "       add one directory to the lookup path for .cdo and .cmi files";
+    "-I", Arg.String (fun s -> Location.obj_path := s::!Location.obj_path),
+      "       add one directory to the lookup path for .cdo/.cmi and include files";
     "--stdin", Arg.Unit (fun () -> src := "" :: !src),
            "  read CDuce script on standard input";
     "--arg", Arg.Rest (fun s -> args := s :: !args),
