@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 type 'a t = Obj.t array
@@ -39,10 +39,16 @@ let map_elements f t =
   in
   aux [] (get_len t - 2)
 
+
+(* TODO: check that the comparison function is necessarily called
+   on a pair of equal elements if such a pair exist.
+   I guess any sort algorithm must do that! *)
+exception DuplicateKey
 let sort a = 
   Array.sort 
     (fun (i,_) (j,_) -> 
-       assert (i != j); if (i:int) < j then (-1) else 1) a
+       if (i == j) then raise DuplicateKey;
+       if (i:int) < j then (-1) else 1) a
 
 let real_create a = 
   let n = Array.length a in
