@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 (* TODO:
@@ -101,7 +101,7 @@ module Printer = struct
     H.add p.ns_to_prefix empty (ref (Set empty_str));
     Table.iter 
       (fun pr ns ->
-	 if (U.get_str pr <> "") then
+	 if (U.get_str pr <> "") && not (Uri.equal empty ns) then
 	   match get_prefix p ns  with
 	     | { contents = Hint l } as r -> 
 		 p.hints <- pr::p.hints; 
