@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 type 'a regexp = 
@@ -413,7 +413,7 @@ module Decompile(H : TABLE)(S : S) = struct
 	then simplify_alt accu rest
 	else simplify_alt (x::accu) rest
 
-  let alt s1 s2 =
+  let rec alt s1 s2 =
     let s1 = match s1 with RAlt x -> x | x -> [x] in
     let s2 = match s2 with RAlt x -> x | x -> [x] in
     let l = merge s1 s2 in
@@ -424,6 +424,8 @@ module Decompile(H : TABLE)(S : S) = struct
     let l = simplify_alt [] l in
     match l with
       | [x] -> x
+      | [ RSeq [ a; RPlus r ] ; a' ] when compare a a' = 0 ->
+	  RSeq [ a ; RStar r ]
       | l -> RAlt l
 
   let rec simplify_seq = function
@@ -503,9 +505,12 @@ module Decompile(H : TABLE)(S : S) = struct
       let s = slot n in
       if not s.ok then (
 	s.ok <- true;
-	let (tr,f) = trans n in
-	if f then add_trans s final epsilon;
-	List.iter (fun (l,dst) -> add_trans s (conv dst) (rtrans l)) tr;
+	match trans n with
+	  | `T (tr,f) ->
+	      if f then add_trans s final epsilon;
+	      List.iter (fun (l,dst) -> add_trans s (conv dst) (rtrans l)) tr
+	  | `Eps (l,dst) ->
+	      add_trans s (conv dst) (alt (rtrans l) epsilon)
       );
       s in
 
