@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 (* Decompilation of regular expressions *)
@@ -32,5 +32,7 @@ end
 
 module Decompile(X : TABLE)(S : S)
 : sig
-  val decompile: (X.key -> (S.t * X.key) list * bool) -> X.key -> S.t regexp
+  val decompile: (X.key -> [ `T of (S.t * X.key) list * bool
+                           | `Eps of S.t * X.key]
+			   ) -> X.key -> S.t regexp
 end
