@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 #load "q_MLast.cmo";;
@@ -643,8 +643,8 @@ let make_wrapper fn =
    with End_of_file -> ());
   let s = gen_wrapper !v in
   !Pcaml.print_implem [ s,_loc ];
-  print_endline "let () = Librarian.obj_path := [";
-  List.iter (fun s -> Printf.printf "  %S;\n" s) !Librarian.obj_path;
+  print_endline "let () = Location.obj_path := [";
+  List.iter (fun s -> Printf.printf "  %S;\n" s) !Location.obj_path;
   print_endline " ];;";
   print_endline "let () = Run.main ();;"
 
