@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 exception Error of string
@@ -235,12 +235,12 @@ let unsupported s =
   raise (Error (Printf.sprintf "Unsupported feature (%s) found in .cmi" s))
 
 let has_cmi name =
-  Config.load_path := Config.standard_library :: !Librarian.obj_path;
+  Config.load_path := Config.standard_library :: !Loc.obj_path;
   try ignore (Misc.find_in_path_uncap !Config.load_path (name ^ ".cmi")); true
   with Not_found -> false
 
 let find_value v =
-  Config.load_path := Config.standard_library :: !Librarian.obj_path;
+  Config.load_path := Config.standard_library :: !Loc.obj_path;
   let li = Longident.parse v in
   ocaml_env := Env.initial;
   let (_,vd) = Env.lookup_value li Env.initial in
@@ -262,7 +262,7 @@ let values_of_sig name sg =
 
 
 let load_module name = 
-  Config.load_path := Config.standard_library :: !Librarian.obj_path;
+  Config.load_path := Config.standard_library :: !Loc.obj_path;
   let li = Longident.parse name in
   ocaml_env := Env.initial;
   let (_,mty) = Env.lookup_module li Env.initial in
@@ -287,7 +287,7 @@ let load_module name =
     raise (Loc.Generic s)
 
 let read_cmi name =
-  Config.load_path := Config.standard_library :: !Librarian.obj_path;
+  Config.load_path := Config.standard_library :: !Loc.obj_path;
   let filename = Misc.find_in_path_uncap !Config.load_path (name ^ ".cmi") in
   let sg = Env.read_signature name filename in
   ocaml_env := Env.add_signature sg Env.initial;
