@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 (* TODO: handle encodings of the input for pretty printing
@@ -171,3 +171,16 @@ let protect_op op =
   if (!protected) then
     raise 
       (Generic (op ^ ": operation not authorized in the web prototype"))
+
+let obj_path = ref [ "" ]
+
+let resolve_filename s =
+  if Filename.is_relative s then
+    try
+      let p = 
+	List.find 
+	  (fun p -> Sys.file_exists (Filename.concat p s)) 
+	  (current_dir () :: !obj_path) in
+      Filename.concat p s
+    with Not_found -> s
+  else s
