@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 (* Locations in source file,
@@ -54,3 +54,7 @@ val loc_of_pos : int * int -> loc
 val set_protected : bool -> unit
 val is_protected : unit -> bool
 val protect_op : string -> unit
+
+
+val obj_path: string list ref
+val resolve_filename: string -> string
