@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 #load "pa_extend.cmo";;
@@ -182,11 +182,8 @@ EXTEND
     | "#"; IDENT "help" -> [ mk _loc (Directive `Help) ]
     | "#"; IDENT "builtins" -> [ mk _loc (Directive `Builtins) ]
     | "include"; s = STRING2 ->
-	let s = 
-	  if Filename.is_relative s 
-	  then Filename.concat (Location.current_dir ()) s
-	  else s in
 	protect_op "File inclusion";
+	let s = Location.resolve_filename s in
 	(* avoid looping; should issue an error ? *)
 	(* it is possible to have looping with x/../x/../x/.. ....
 	   Need to canonicalize filename *)
@@ -231,7 +228,7 @@ EXTEND
 	| "transform" | "fun" | "in"
 	| "let" | "type" | "debug" | "include"
         | "and" | "or" | "validate" | "schema" | "namespace" | "ref" | "alias"
-	| "not" | "as" | "where" | "select" | "from"
+	| "not" | "as" | "where" | "select" | "from" | "open"
 	]
 	-> a
     ]
@@ -276,7 +273,7 @@ EXTEND
     | "not"; e = expr -> exp _loc (logical_not e)
     ]
     |
-    [ e1 = expr; ":="; e2 = expr -> exp _loc (set_ref e1 e2)
+    [ e1 = expr; ":="; e2 = expr -> exp _loc  (set_ref e1 e2)
     ]
     | 
     [ e1 = expr; op = ["=" | "<=" | "<<" | ">>" | ">=" ]; e2 = expr -> 
@@ -317,22 +314,37 @@ EXTEND
         let t = (p, Pair (Var id_dummy,cst_nil)) in
         exp _loc (Transform (e,[t]))
      | e = expr; "//" ; p = pat ->
-	 (*
-	   let $stack=ref [p*] [] in
-	   let _ = xtransform e with $$$ & p -> $stack := !$stack @ $$$ in
-	   !stack;; 
-	 *)
-	 let stk = U.mk "$stack" in
+	 (* let $xstack=ref [p*] [] in                  
+            let f  ( x : [Any*]) : [Any*] = 
+                     xtransform x with $$$ & p & <_ ..>[y::Any* ] ->  $stack := !$stack @ [$$$] ; f y
+            in let _ =f e in !$stack *)
+
+	 let stk = U.mk "$stack" in 
+	 let y = U.mk "y" in
+	 let x = U.mk "x" in
+	 let f = U.mk "f" in
 	 let assign = 
 	   set_ref 
 	     (Var stk)
-	     (concat (get_ref (Var stk)) (Pair (Var id_dummy,cst_nil))) in
-	 let capt = mk _loc (And (mk _loc (PatVar [U.mk "$$$"]),p)) in
-	 let xt = Xtrans (e,[capt,assign]) in
-	 let rf = Ref (cst_nil, mk _loc (Regexp (Star(Elem p)))) in
+	     (concat (get_ref (Var stk)) (Pair (Var id_dummy,cst_nil))) in  
+        	let tag = mknoloc (Internal (Types.atom (Atoms.any))) in
+	        let att = mknoloc (Internal Types.Record.any) in
+	        let any = mknoloc (Internal Types.any) in
+	        let re = (SeqCapture(noloc,y,Star(Elem(any)))) in
+	        let ct = mknoloc (Regexp re) in
+                let children = mknoloc (XmlT (tag, multi_prod _loc [att;ct])) in
+         let capt = mknoloc (And (mknoloc (And (mknoloc (PatVar [id_dummy]),p)),children)) in  
+	 let assign = seq assign ( (Apply(Var(f) , Var(y) ) ) ) in
+	 let xt = Xtrans ((Var x),[capt,assign]) in 
+	 let rf = Ref (cst_nil, mknoloc (Regexp (Star(Elem p)))) in 
+         let targ = mknoloc (Regexp(Star(Elem(any)))) in
+	 let tres = targ in
+	 let arg = mknoloc(PatVar [x]) in
+         let abst = {fun_name = Some (lop _loc,ident "f") ; fun_iface = [(targ, tres)] ;fun_body = [(arg,xt)] } in
 	 let body = 
-	   let_in rf (mk _loc (PatVar [stk]))
-	     (let_in xt (mk _loc (Internal Types.any)) (get_ref (Var stk)))
+	   let_in rf (mknoloc (PatVar [stk]))
+	    (let_in ((Abstraction abst)) (mknoloc (PatVar[ident "f"])) 
+              (let_in ((Apply(Var(f) , e) )  ) (mknoloc (Internal Types.any)) (get_ref (Var stk))))
 	 in
 	 exp _loc body
     ]
@@ -477,7 +489,7 @@ EXTEND
 		 others (tres,body) in
 	     let (targ,arg) = mkfun ((p1,targ1) :: args) in
 	     [(targ,tres)],[(arg,body)]
-	      ] ];
+	      ] ]; 
 
 
  fun_decl: [
