@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 let buflen = 1024
@@ -14,7 +14,7 @@ let load_from_file p s =
       let msg = 
 	Printf.sprintf "load_xml, file \"%s\": %s" s (Printexc.to_string exn)
       in
-      raise (Location.Generic msg)
+      Value.failwith' msg
   in
   let rec loop () =
     let n = input ic buf 0 buflen in
