@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 (* Print XML documents *)
@@ -19,13 +19,13 @@ let write_markup_string ~to_enc buf s =
 	      failwith ("Cannot represent code point " ^ string_of_int n))
     s
   in
-  Buffer.add_string buf s'
+  buf s'
 
 let write_data_string ~to_enc buf s =
   let write_part i len =
     if (len > 0) then
       if to_enc = `Enc_utf8 
-      then Buffer.add_substring buf s i len
+      then buf (String.sub s i len)
       else
 	let s' = 
 	  convert
@@ -34,7 +34,7 @@ let write_data_string ~to_enc buf s =
             ~subst:(fun n -> "&#" ^ string_of_int n ^ ";")
 	    ~range_pos:i ~range_len:len s
 	in
-	Buffer.add_string buf s'
+	buf s'
   in
   let i = ref 0 in
   for k = 0 to String.length s - 1 do
@@ -42,11 +42,11 @@ let write_data_string ~to_enc buf s =
       | ('&' | '<' | '>' | '"' | '%') as c ->
           write_part !i (k - !i);
           begin match c with
-              '&' -> Buffer.add_string buf "&amp;"
-            | '<' -> Buffer.add_string buf "&lt;"
-            | '>' -> Buffer.add_string buf "&gt;"
-            | '"' -> Buffer.add_string buf "&quot;"
-            | '%' -> Buffer.add_string buf "&#37;"  (* reserved in DTDs *)
+              '&' -> buf "&amp;"
+            | '<' -> buf "&lt;"
+            | '>' -> buf "&gt;"
+            | '"' -> buf "&quot;"
+            | '%' -> buf "&#37;"  (* reserved in DTDs *)
             | _   -> assert false
           end;
           i := k+1
@@ -83,7 +83,7 @@ let rec schema_value ?(recurs=true) ~wds v = match v with
   | v when Value.equal v Value.vtrue -> wds true_literal
   | v when Value.equal v Value.vfalse -> wds false_literal
   | Pair _ as v when recurs -> schema_values ~wds v
-  | String_utf8 _ as v -> wds (fst (get_string_utf8 v))
+  | String_utf8 _ | String_latin1 _ as v -> wds (fst (get_string_utf8 v))
   | _ -> raise exn_print_xml
 
 and schema_values ~wds v =
@@ -96,10 +96,9 @@ and schema_values ~wds v =
       schema_values ~wds tl
   | _ -> raise exn_print_xml
 
-let string_of_xml ~utf8 ns_table v = 
+let to_buf ~utf8 buffer ns_table v = 
   let to_enc = if utf8 then `Enc_utf8 else `Enc_iso88591 in
 
-  let buffer = Buffer.create 127 in
   let printer = Ns.Printer.printer ns_table in
 
   let wms = write_markup_string ~to_enc buffer
@@ -187,10 +186,14 @@ let string_of_xml ~utf8 ns_table v =
       | v -> schema_value ~wds v
   in
   document_start ();
-  print_elt (Ns.Printer.prefixes printer) v;
-  Buffer.contents buffer
+  print_elt (Ns.Printer.prefixes printer) v
 
 let print_xml ~utf8 ns_table s =
-  let s = string_of_xml ~utf8 ns_table s in
+  let buf = Buffer.create 32 in
+  to_buf ~utf8 (Buffer.add_string buf) ns_table s;
+  let s = Buffer.contents buf in
   if utf8 then string_utf8 (U.mk s) else string_latin1 s
  
+let dump_xml  ~utf8 ns_table s =
+  to_buf ~utf8 print_string ns_table s;
+  Value.nil
