@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 open Operators
@@ -40,7 +40,10 @@ end
 
 let run_process cmd =
   let (sout,sin,serr) as h = Unix.open_process_full cmd (Unix.environment()) in
-  close_out sin;
+  Unix.close (Unix.descr_of_out_channel sin);
+  (* used to be: (close_out sin), but OCaml 3.09.2 seems to segfault
+     on double closing channels. *)
+
   let sout = Reader.ic sout in
   let serr = Reader.ic serr in
   sout,serr, Unix.close_process_full h
