@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 open Ident
@@ -276,7 +276,7 @@ let rec print ppf v =
   else  match v with
     | Pair (x,y) -> Format.fprintf ppf "(%a,%a)" print x print y
     | Xml (x,y,z) | XmlNs (x,y,z,_) -> print_xml ppf x y z
-    | Record l -> Format.fprintf ppf "{%a }" print_record (Imap.elements l)
+    | Record l -> Format.fprintf ppf "@[{%a }@]" print_record (Imap.elements l)
     | Atom a ->	Atoms.V.print_quote ppf a
     | Integer i -> Intervals.V.print ppf i
     | Char c ->	Chars.V.print ppf c
@@ -347,7 +347,7 @@ and print_attr ppf = function
 
 and print_record ppf = function
   | [] -> ()
-  | f :: rem -> Format.fprintf ppf " %a" print_field f; print_record ppf rem
+  | f :: rem -> Format.fprintf ppf "@ %a" print_field f; print_record ppf rem
 
 and print_field ppf (l,v) = 
   Format.fprintf ppf "%a=%a" Label.print_attr (Label.from_int l) print v  
