@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 open Printf
@@ -137,7 +137,8 @@ let stream_of_value v =
   let f _ = (* lazy visit of a tree of CDuce XML values, stack keeps track of
             what has still to be visited *)
     match !stack with
-    | (Fully ((Value.Xml (Value.Atom atom, attrs, _)) as v)) :: tl ->
+    | (Fully ((Value.Xml (Value.Atom atom, attrs, _))
+       |(Value.XmlNs (Value.Atom atom, attrs, _, _)) as v)) :: tl ->
         stack := (Half v) :: tl;
         let children = ref [] in  (* TODO inefficient *)
         let push v s = (s := v :: !s) in
@@ -145,7 +146,7 @@ let stream_of_value v =
           (fun pcdata -> push (Other pcdata) children)
           (fun v ->
             match v with
-            | (Value.Xml (_, _, _)) as v -> push (Fully v) children
+            | (Value.Xml _ | Value.XmlNs _) as v -> push (Fully v) children
             | v -> raise (Invalid_argument "Schema_events.stream_of_value"))
           v;
         stack := (List.rev !children) @ !stack;
