@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 open Encodings
@@ -69,6 +69,10 @@ let print s = match get s with
 
 type 'a map = 'a Imap.t * 'a Imap.t * 'a option
 
+let map_map f (m1,m2,o) =
+  Imap.map f m1, Imap.map f m2, 
+  (match o with Some x -> Some (f x) | None -> None)
+
 (* TODO: optimize this get_map *)
 let get_map q (mtags,mns,def) =   
   try Imap.find mtags (Upool.int q)
@@ -101,3 +105,12 @@ let mk_map l =
   let mtags = Imap.create (Array.of_list !all_tags) in
   let mns = Imap.create (Array.of_list !all_ns) in
   (mtags,mns,!def)
+
+type sample = (Ns.Uri.t * Ns.Label.t option) option
+
+let contains_sample s t =
+  match s,(get t) with
+    | None, `Cofinite _ -> true
+    | None, `Finite _ -> false
+    | Some (_,Some tag),_ -> contains tag t
+    | Some (ns, None),_ -> is_empty (diff (any_in_ns ns) t)
