@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 open Encodings
@@ -38,7 +38,11 @@ val print_tag : t -> (Format.formatter -> unit) option
 
 val single : t -> V.t
 
+type sample = (Ns.Uri.t * Ns.Label.t option) option
+val sample : t -> sample
+val contains_sample: sample -> t -> bool
+
 type 'a map
 val mk_map: (t * 'a) list -> 'a map
 val get_map: V.t -> 'a map -> 'a
-
+val map_map: ('a -> 'b) -> 'a map -> 'b map
