@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 open Builtin_defs
@@ -31,6 +31,7 @@ let types =
     "Float",   float;
     "AnyXml",  any_xml;
     "Namespaces", namespaces;
+    "Caml_int", caml_int;
   ]
 
 let env =
@@ -83,6 +84,9 @@ let unary_op_warning name targ w t run =
 
 open Ident
 
+let raise_gen exn =
+  raise (Value.CDuceExn (Value.string_latin1 (Printexc.to_string exn)))
+
 let exn_load_file_utf8 = lazy (
   Value.CDuceExn (
     Value.Pair (
@@ -217,6 +221,19 @@ register_fun "print_xml_utf8"
   (fun v -> Print_xml.print_xml ~utf8:true !Eval.ns_table v);;
 
 
+register_fun "dump_xml" 
+  Types.any nil
+  (fun v ->
+     Location.protect_op "print"; 
+     Print_xml.dump_xml ~utf8:false !Eval.ns_table v);;
+
+register_fun "dump_xml_utf8" 
+  Types.any nil
+  (fun v -> 
+     Location.protect_op "print";
+     Print_xml.dump_xml ~utf8:true !Eval.ns_table v);;
+
+
 register_fun "print"
   string_latin1 nil
   (fun v ->
@@ -294,22 +311,24 @@ register_fun "make_atom"
 
 binary_op_warning2 "dump_to_file"
   string_latin1 string string_latin1 nil
-  (fun f v ->
+  (fun f v -> try
      Location.protect_op "dump_to_file";
      let oc = open_out (Value.get_string_latin1 f) in
      output_string oc (Value.get_string_latin1 v);
      close_out oc;
-     Value.nil);;
+     Value.nil
+   with exn -> raise_gen exn);;
     
 binary_op_cst "dump_to_file_utf8"
   string_latin1 string nil
-  (fun f v ->
+  (fun f v -> try
      Location.protect_op "dump_to_file_utf8";
      let oc = open_out (Value.get_string_latin1 f) in
      let (v,_) = Value.get_string_utf8 v in
      output_string oc (U.get_str v);
      close_out oc;
-     Value.nil);;
+     Value.nil
+   with exn -> raise_gen exn);;
 
 (* Integer operators *)
 
@@ -363,8 +382,7 @@ binary_op_warning2 "mod"
 
 binary_op_gen "@"
   (fun arg1 arg2 constr precise ->
-     let constr' = Sequence.star 
-		     (Sequence.approx (Types.cap Sequence.any constr)) in
+     let constr' = Sequence.ub_concat constr in
      let exact = Types.subtype constr' constr in
      if exact then
        let t1 = arg1 constr' precise
