@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 module V = struct
@@ -130,7 +130,16 @@ let print =
 	 Format.fprintf ppf "%a--%a" V.print a V.print b
     )
 
+let dump ppf t =
+  match print t with
+    | [] -> Format.fprintf ppf "()"
+    | hd::tl -> hd ppf; List.iter (fun x -> Format.fprintf ppf "|"; x ppf) tl
+  
+
 type 'a map = (int * 'a) list
+
+let map_map f l = List.map (fun (i,x) -> (i, f x)) l
+
 (* Optimize lookup:
    - decision tree
    - merge adjacent segment with same result
