@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 module V : sig
@@ -17,6 +17,8 @@ end
 
 include Custom.T
 val print : t -> (Format.formatter -> unit) list
+val dump: Format.formatter -> t -> unit
+
 
 val empty : t
 val any   : t
@@ -38,3 +40,4 @@ val single : t -> V.t
 type 'a map
 val mk_map: (t * 'a) list -> 'a map
 val get_map: V.t-> 'a map -> 'a
+val map_map: ('a -> 'b) -> 'a map -> 'b map
