@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 exception Error of string
@@ -505,7 +505,9 @@ module Normal = struct
       List.iter (fun p -> assert(Types.subtype t (Types.descr p.accept)))
 	(NodeSet.get pl)
     let print ppf (pl,t,xs) =
-      Format.fprintf ppf "@[(pl=%a;t=%a)@]" NodeSet.dump pl Types.Print.print t
+      Format.fprintf ppf "@[(pl=%a;t=%a;xs=%a)@]" 
+	NodeSet.dump pl Types.Print.print t
+	IdSet.dump xs
     let compare (l1,t1,xs1) (l2,t2,xs2) =
       let c = NodeSet.compare l1 l2 in if c <> 0 then c
       else let c = Types.compare t1 t2 in if c <> 0 then c
@@ -651,8 +653,9 @@ module Normal = struct
 		      (if y then Some empty_res else None))
       | Some l ->
 	  RecLabel (l,aux (Types.Record.split_normal t l)) in
-    {  nprod = aux (Types.Product.normal t);
-       nxml  = aux (Types.Product.normal ~kind:`XML t);
+    {  nprod = aux (Types.Product.clean_normal (Types.Product.normal t));
+       nxml  = 
+	aux (Types.Product.clean_normal (Types.Product.normal ~kind:`XML t));
        nrecord = record
     }
 
@@ -1004,7 +1007,7 @@ module Compile = struct
 	    "IFACE=%a@." print_iface d.interface; 
 	  for i = 0 to Array.length a - 1 do
 	    Format.fprintf Format.std_formatter
-	      "a.(i)=%b@." (a.(i) != None)
+	      "a.(%i)=%b@." i (a.(i) != None)
 	  done;
 	  assert false
       | `Switch (yes,_) when a.(i) != None -> aux (i + 1) yes
@@ -1091,8 +1094,15 @@ module Compile = struct
   let get_tests facto pl f t d post =
     let pl = Array.map (List.map f) pl in
 
+    let factorized = ref NfMap.empty in
     (* Collect all subrequests *)
-    let aux reqs (req,_) = NfSet.add req reqs in
+    let aux reqs (req,_) = 
+      let (_,_,((_,tr,xs) as r')) as req' =
+	if facto then Normal.factorize t req else [],[],req in
+      factorized := NfMap.add req req' !factorized;
+      
+      if IdSet.is_empty xs && Types.subtype t tr then reqs
+      else NfSet.add r' reqs in
     let reqs = Array.fold_left (List.fold_left aux) NfSet.empty pl in
     let reqs = Array.of_list (NfSet.elements reqs) in
 
@@ -1102,25 +1112,83 @@ module Compile = struct
     let idx = !idx in
 
     (* Build dispatcher *)
-    let reqs_facto =
-      if facto then Array.map (Normal.factorize t) reqs
-      else Array.map (fun r -> [],[],r) reqs in
-    let reqs = Array.map (fun (_,_,req) -> req) reqs_facto in
-
-    let disp = dispatcher t reqs in
+(*    if Array.length reqs = 0 then print_endline "NOREQ!"; *)
+    let disp = dispatcher 
+      (if Array.length reqs = 0 then Types.Record.any_or_absent else t) reqs in
     
     (* Build continuation *)
     let result (t,ar,m) =
       let get (req,info) a =
-	let i = NfMap.find req idx in
-	let (var,nil,_) = reqs_facto.(i) in
-	match m.(i) with Some res -> ((var,nil,res),info)::a | _ -> a in
+	let (var,nil,r') = 
+	  try NfMap.find req !factorized 
+	  with Not_found -> assert false in
+	try
+	  let i = NfMap.find r' idx in
+	  match m.(i) with Some res -> ((var,nil,res),info)::a | _ -> a 
+	with Not_found ->
+	  ((var,nil,IdMap.empty),info)::a
+      in
       let pl = Array.map (fun l -> List.fold_right get l []) pl in
       d t ar pl
     in
     let res = Array.map result disp.codes in
     post (disp,res)
 
+   let add_factorized disp rhs =
+     let result ((code,srcs,pop) as r) =
+       match rhs.(code) with
+	 | Fail -> r
+	 | Match (_,(var,nil,xs,_)) ->
+	     let pos = ref (-1) in
+	     let var x =
+	       if IdSet.mem var x then Catch
+	       else if IdSet.mem nil x then Nil
+	       else (incr pos; srcs.(!pos)) in
+	     let srcs' = Array.of_list (List.map var (IdSet.get xs)) in
+	     assert(succ !pos = Array.length srcs);
+	     (code,srcs',pop) in
+     let dispatch1 = function
+       | Dispatch (s,a) -> Dispatch (s, Array.map result a)
+       | TailCall s -> 
+	   let f code (_,ar,_) =
+	     let srcs = Array.init ar (fun i -> Stack (ar - i)) in
+	     result (code,srcs,ar) in
+	   Dispatch (s, Array.mapi f disp.codes)
+       | Ignore r -> Ignore (result r)
+       | Impossible -> Impossible in
+     let dispatch2 = function
+       | Dispatch (s,a) -> Dispatch (s, Array.map dispatch1 a)
+       | TailCall s -> 
+	   let f code (_,ar,_) =
+	     let srcs = Array.init ar (fun i -> Stack (ar - i)) in
+	     Ignore (result (code,srcs,ar)) in
+	   Dispatch (s, Array.mapi f disp.codes)
+       | Ignore r -> Ignore (dispatch1 r)
+       | Impossible -> Impossible in
+     let state = disp.state in
+     let actions = match state.actions with
+       | AIgnore r -> AIgnore (result r)
+       | AKind k -> 
+	   AKind  {
+	     basic = List.map (fun (t,r) -> (t,result r)) k.basic;
+	     atoms = Atoms.map_map result k.atoms;
+	     chars = Chars.map_map result k.chars;
+	     prod = dispatch2 k.prod;
+	     xml = dispatch2 k.xml;
+	     record = 
+	       (match k.record with
+		  | None -> None
+		  | Some (RecLabel (l,x)) -> Some (RecLabel (l,dispatch2 x))
+		  | Some (RecNolabel (x,y)) ->
+		      Some (RecNolabel (
+			      (match x with None -> None 
+				 | Some r -> Some (result r)),
+			      (match y with None -> None 
+				 | Some r -> Some (result r)))))
+	   }
+     in
+     { state with actions = actions }
+
   let make_branches t brs =
     let t0 = ref t in
     let aux (p,e) = 
@@ -1129,17 +1197,26 @@ module Compile = struct
       let nnf = (Normal.NodeSet.singleton p, Types.cap !t0 tp, xs) in
       t0 := Types.diff !t0 tp;
       [(nnf, (xs, e))] in
+    let has_facto = ref false in
     let res _ _ pl =
       let aux r = function 
-	| [(([],[],res), (xs,e))] -> assert (r == Fail); 
+	| [((var,nil,res), (xs,e))] -> assert (r == Fail); 
 	    let i = ref 0 in
-	    List.iter (fun x -> assert (IdMap.assoc x res = !i); incr i) xs;
-	    Match (List.length xs, e)
+	    List.iter (fun x -> 
+			 if IdSet.mem var x || IdSet.mem nil x 
+			 then has_facto := true
+			 else (assert (IdMap.assoc x res = !i); incr i)) xs;
+	    Match (List.length xs, (var,nil,xs,e))
 	| [] -> r | _ -> assert false in
       Array.fold_left aux Fail pl in
+(*    Format.fprintf Format.std_formatter
+      "make_branches t=%a   #branches=%i@." Types.Print.print t (List.length brs);  *)
     let pl = Array.map aux (Array.of_list brs) in
-    get_tests false pl (fun x -> x) t res (fun (disp,rhs) -> disp.state,rhs)
-
+    let disp,rhs = get_tests true pl (fun x -> x) t res (fun x -> x) in
+    let state = if !has_facto then add_factorized disp rhs else disp.state in
+    state,
+    Array.map 
+      (function Match (n,(_,_,_,e)) -> Match (n,e) | Fail -> Fail) rhs
 
   let rec dispatch_prod0 disp t pl =
     get_tests true pl
@@ -1159,12 +1236,14 @@ module Compile = struct
 
   let dispatch_prod disp pl =
     let t = Types.Product.get disp.t in
+    if t == [] then Impossible else 
     dispatch_prod0 disp t 
       (Array.map (fun p -> Normal.NLineProd.elements p.Normal.nprod) pl) 
 (*    dispatch_prod0 disp t (Array.map Normal.prod_tests disp.pl)  *)
 
   let dispatch_xml disp pl = 
     let t = Types.Product.get ~kind:`XML disp.t in
+    if t == [] then Impossible else 
     dispatch_prod0 disp t 
       (Array.map (fun p -> Normal.NLineProd.elements p.Normal.nxml) pl)
 
@@ -1291,7 +1370,7 @@ module Compile = struct
     let (d,rhs) = make_branches t0 [ (p,()) ] in
     let code = ref (-1) in
     Array.iteri 
-      (fun (i : int) (rhs : unit rhs) -> 
+      (fun (i : int) rhs -> 
 	 match rhs with
 	   | Fail -> assert (!code < 0); code := i | _ -> ()) rhs;
     if (!code >= 0) then prepare_checker !code d;
