@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 type error = 
@@ -24,6 +24,15 @@ module V = Types.Positive
 module H = Map.Make(Types)
 module H2 = Map.Make(Custom.Pair(Types)(Types))
 
+let memoize f =
+  let memo = ref H.empty in
+  fun t ->
+    try H.find t !memo
+    with Not_found ->
+      let z = f t in
+      memo := H.add t z !memo;
+      z
+
 let mapping f t queue =
   let memo = ref H.empty in
   let rec aux t =
@@ -38,16 +47,18 @@ let mapping f t queue =
       v
   in
   aux t
-  
 
 let aux_concat = mapping (fun t v -> V.times (V.ty t) v)
 let aux_flatten t = mapping aux_concat t (V.ty nil_type)
-let aux_map f t = mapping (fun t v -> V.times (V.ty (f t)) v) t (V.ty nil_type)
+let aux_map f t = 
+  let f = memoize f in 
+  mapping (fun t v -> V.times (V.ty (f t)) v) t (V.ty nil_type)
 
 let solve x = Types.descr (V.solve x)
 
+
 let concat t1 t2 = solve (aux_concat t1 (V.ty t2))
-let flatten t = solve (aux_flatten t)
+let flatten t = let t = Types.normalize t in solve (aux_flatten t)
 let map f t = solve (aux_map f t)
 
 
@@ -238,3 +249,4 @@ let any_xtransformable =
 	  let t = Types.cons (Types.cup elt non_elt) in
 	  Types.cup nil_type (Types.times t n)))
 
+let ub_concat t = star (approx (Types.cap any t))
