@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 type error = 
@@ -25,7 +25,6 @@ val concat: Types.t -> Types.t -> Types.t
 val flatten: Types.t -> Types.t
 val map: (Types.t -> Types.t) -> Types.t -> Types.t
 
-
 val map_tree: 
   Types.t -> (Types.t -> Types.t -> Types.t * Types.t) -> Types.t -> Types.t
  (* maximal output type -> input type -> (result, residual) *)         
@@ -55,6 +54,9 @@ val approx: Types.t -> Types.t
       t <= (X where X = (s, X) |  Any \ (Any,Any))
   *)
 
+val ub_concat: Types.t -> Types.t
+  (* Returns  [star (approx t)] *)
+
 (* Added this interface needed in cdo2cmo -- Julien *)
 val star_node : Types.Node.t -> Types.Node.t
 
