@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 module Make(X : Custom.T) = struct
@@ -416,6 +416,7 @@ module type FiniteCofinite = sig
   val contains: elem -> t -> bool
   val disjoint: t -> t -> bool
   val is_empty: t -> bool
+  val sample: t -> elem option
 end
 
 module FiniteCofinite(X : Custom.T) = struct
@@ -444,6 +445,11 @@ module FiniteCofinite(X : Custom.T) = struct
     | Finite s -> Format.fprintf ppf "Finite[%a]" SList.dump s
     | Cofinite s -> Format.fprintf ppf "Cofinite[%a]" SList.dump s
 
+  let sample = function
+    | Finite (x::_) -> Some x
+    | Finite [] -> raise Not_found
+    | Cofinite _ -> None
+
 
   let empty = Finite []
   let any = Cofinite []
@@ -497,6 +503,12 @@ struct
   module T = T0.Map
   type t = Finite of TMap.t | Cofinite of TMap.t
 
+  let sample = function
+    | Cofinite _ -> None
+    | Finite l -> (match T.get l with 
+		     | [] -> raise Not_found
+		     | (x,y)::_ -> Some (x, SymbolSet.sample y))
+
   let get = function
     | Finite l -> `Finite (T.get l)
     | Cofinite l -> `Cofinite (T.get l)
