@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 module Make(X : Custom.T) :
@@ -97,6 +97,8 @@ module type FiniteCofinite = sig
   val contains: elem -> t -> bool
   val disjoint: t -> t -> bool
   val is_empty: t -> bool
+
+  val sample: t -> elem option
 end
 
 module FiniteCofinite(X : Custom.T) : FiniteCofinite with type elem = X.t
@@ -120,4 +122,7 @@ sig
 
   val get: t -> [ `Finite of (X.t * SymbolSet.t) list 
 		| `Cofinite of (X.t * SymbolSet.t) list ]
+
+  val sample: t -> (X.t * SymbolSet.elem option) option
+
 end
