@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 open Ident
@@ -101,6 +101,12 @@ struct
 	  List.iter (fun x -> Format.fprintf ppf " \\@ !%s" x) l;
 	  Format.fprintf ppf "@]" ]
 
+  let contains_sample s t = match s,t with
+    | None, Cofinite _ -> true
+    | None, Finite _ -> false
+    | Some s, t -> contains s t
+    
+
 end
 
 
@@ -486,153 +492,264 @@ let rec exists max f =
 
 exception NotEmpty
 
+module Witness = struct
+
+  module NodeSet = Set.Make(Node)
+
+  type witness =
+    | WInt of Intervals.V.t
+    | WAtom of Atoms.sample
+    | WChar of Chars.V.t
+    | WAbsent
+    | WAbstract of Abstract.elem option
+
+    | WPair of witness * witness * witness_slot
+    | WXml of witness * witness * witness_slot
+    | WRecord of witness label_map * bool * witness_slot
+	(* Invariant: WAbsent cannot actually appear *)
+
+    | WFun of (witness * witness option) list * witness_slot
+  and witness_slot = 
+      { mutable wnodes_in: NodeSet.t;
+	mutable wnodes_out: NodeSet.t;
+	mutable wuid: int }
+
+  module WHash = Hashtbl.Make(
+    struct
+      type t = witness
+      let hash_small = function
+	| WInt i -> 17 * Intervals.V.hash i
+	| WChar c -> 1 + 17 * Chars.V.hash c
+	| WAtom None -> 2
+	| WAtom (Some (ns,None)) -> 3 + 17 * Ns.Uri.hash ns
+	| WAtom (Some (_,Some t)) -> 4 + 17 * Ns.Label.hash t
+	| WAbsent -> 5
+	| WAbstract None -> 6
+	| WAbstract (Some t) -> 7 + 17 * Abstract.T.hash t
+	| WPair (_,_,s) 
+	| WXml (_,_,s)
+	| WRecord (_,_,s)
+	| WFun (_,s) -> 8 + 17 * s.wuid
+      let hash = function
+	| WPair (p1,p2,_) -> 257 * hash_small p1 + 65537 * hash_small p2
+	| WXml (p1,p2,_) -> 1 + 257 * hash_small p1 + 65537 * hash_small p2
+	| WRecord (r,o,_) -> 
+	    (if o then 2 else 3) + 257 * LabelMap.hash hash_small r
+	| WFun (f,_) ->
+	    4 + 257 *
+	      (Hashtbl.hash 
+		 (List.map 
+		    (function (x,None) -> 17 * hash_small x
+		       | (x,Some y) -> 
+			   1 + 17 * hash_small x + 257 * hash_small y)
+		    f)
+	      )
+	| _ -> assert false
+
+      let equal_small w1 w2 = match w1,w2 with
+	| WInt i1, WInt i2 -> Intervals.V.equal i1 i2
+	| WChar c1, WChar c2 -> Chars.V.equal c1 c2
+	| WAtom None, WAtom None -> true
+	| WAtom (Some (ns1,None)), WAtom (Some (ns2,None)) ->
+	    Ns.Uri.equal ns1 ns2
+	| WAtom (Some (_,Some t1)), WAtom (Some (_,Some t2)) ->
+	    Ns.Label.equal t1 t2
+	| WAbsent, WAbsent -> true
+	| WAbstract None, WAbstract None -> false
+	| WAbstract (Some t1), WAbstract (Some t2) -> Abstract.T.equal t1 t2
+	| _ -> w1 == w2
+
+      let equal w1 w2 = match w1,w2 with
+	| WPair (p1,q1,_), WPair (p2,q2,_) 
+	| WXml (p1,q1,_), WXml (p2,q2,_) -> 
+	    equal_small p1 p2 && equal_small q1 q2
+	| WRecord (r1,o1,_), WRecord (r2,o2,_) ->
+	    o1 == o2 && (LabelMap.equal equal_small r1 r2)
+	| WFun (f1,_), WFun (f2,_) ->
+	    List.length f1 = List.length f2 &&
+		List.for_all2
+		(fun (x1,y1) (x2,y2) ->
+		   equal_small x1 x2 && (match y1,y2 with
+					   | Some y1, Some y2 -> 
+					       equal_small y1 y2
+					   | None, None -> true
+					   | _ -> false)
+		) f1 f2
+	| _ -> false
+    end)
+
+  let wmemo = WHash.create 1024
+  let wuid = ref 0
+  let wslot () = { wuid = !wuid; wnodes_in = NodeSet.empty; 
+		   wnodes_out = NodeSet.empty }
+
+
+  let () =
+    Stats.register Stats.Summary
+      (fun ppf -> Format.fprintf ppf "Allocated witnesses:%i@\n" !wuid)
+
+
+  let rec print_witness ppf = function
+    | WInt i ->
+	Format.fprintf ppf "%a" Intervals.V.print i
+    | WChar c ->
+	Format.fprintf ppf "%a" Chars.V.print c
+    | WAtom None ->
+	Format.fprintf ppf "`#:#"
+    | WAtom (Some (ns,None)) ->
+	Format.fprintf ppf "`%a" Ns.InternalPrinter.print_any_ns ns
+    | WAtom (Some (_,Some t)) ->
+	Format.fprintf ppf "`%a" Ns.Label.print_attr t
+    | WPair (w1,w2,_) -> 
+	Format.fprintf ppf "(%a,%a)" print_witness w1 print_witness w2
+    | WXml (w1,w2,_) -> 
+	Format.fprintf ppf "XML(%a,%a)" print_witness w1 print_witness w2
+    | WRecord (ws,o,_) ->
+	Format.fprintf ppf "{";
+	LabelMap.iteri
+	  (fun l w -> Format.fprintf ppf " %a=%a" 
+	     Label.print_attr l print_witness w)
+	  ws;
+	if o then Format.fprintf ppf " ..";
+	Format.fprintf ppf " }"
+    | WFun (f,_) ->
+	Format.fprintf ppf "FUN{";
+	List.iter (fun (x,y) ->
+		     Format.fprintf ppf " %a->" print_witness x;
+		     match y with
+		       | None -> Format.fprintf ppf "#"
+		       | Some y -> print_witness ppf y) f;
+	Format.fprintf ppf " }"
+    | WAbstract None ->
+	Format.fprintf ppf "Abstract(..)"
+    | WAbstract (Some s) ->
+	Format.fprintf ppf "Abstract(%s)" s
+    | WAbsent ->
+	Format.fprintf ppf "Absent"
+	  
+  let wmk w =  (* incr wuid; w *)  (* hash-consing disabled *)
+    try WHash.find wmemo w
+    with Not_found -> 
+      incr wuid; 
+      WHash.add wmemo w w;
+(*      Format.fprintf Format.std_formatter "W:%a@." 
+	print_witness w; *)
+      w
+
+  let wpair p1 p2 = wmk (WPair (p1,p2, wslot()))
+  let wxml p1 p2 = wmk (WXml (p1,p2, wslot()))
+  let wrecord r o = wmk (WRecord (r,o, wslot()))
+  let wfun f = wmk (WFun (f, wslot()))
+
+  let bool_pair f =
+    BoolPair.compute 
+      ~empty:false ~full:true 
+      ~cup:(||) ~cap:(&&) ~diff:(fun x y -> x && not y) 
+      ~atom:f
+
+  let bool_rec f =
+    BoolRec.compute 
+      ~empty:false ~full:true 
+      ~cup:(||) ~cap:(&&) ~diff:(fun x y -> x && not y) 
+      ~atom:f
+
+  let rec node_has n = function
+    | WXml (_,_,s) | WPair (_,_,s) | WFun (_,s) | WRecord (_,_,s) as w ->
+	if NodeSet.mem n s.wnodes_in then true
+	else if NodeSet.mem n s.wnodes_out then false
+	else (let r = type_has (descr n) w in
+	      if r then s.wnodes_in <- NodeSet.add n s.wnodes_in
+	      else s.wnodes_out <- NodeSet.add n s.wnodes_out;
+	      r)
+    | w -> type_has (descr n) w
+
+  and type_has t = function
+    | WInt i -> Intervals.contains i t.ints
+    | WChar c -> Chars.contains c t.chars
+    | WAtom a -> Atoms.contains_sample a t.atoms
+    | WPair (w1,w2,_) -> 
+	bool_pair 
+	  (fun (n1,n2) -> node_has n1 w1 && node_has n2 w2) 
+	  t.times
+    | WXml (w1,w2,_) ->
+	bool_pair 
+	  (fun (n1,n2) -> node_has n1 w1 && node_has n2 w2)
+	  t.xml
+    | WFun (f,_) ->
+	bool_pair 
+	  (fun (n1,n2) ->
+	     List.for_all
+	       (fun (x,y) ->
+		  not (node_has n1 x) ||
+		    (match y with None -> false
+		       | Some y -> node_has n2 y))
+	       f) 
+	  t.arrow
+    | WRecord (f,o,_) ->
+	bool_rec 
+	  (fun (o',f') ->
+	     ((not o) || o') && (
+	       let checked = ref 0 in
+	       try 
+		 LabelMap.iteri 
+		   (fun l n ->
+		      let w = 
+			try let w = LabelMap.assoc l f in incr checked; w
+			with Not_found -> WAbsent in
+		      if not (node_has n w) then raise Exit
+		   ) f'; 
+		 o' || (LabelMap.length f == !checked)
+		   (* All the remaining fields cannot be WAbsent
+		      because of an invariant. Otherwise, we must
+		      check that all are WAbsent here. *)
+	       with Exit -> false))
+	  t.record
+    | WAbsent -> t.absent
+    | WAbstract a -> Abstract.contains_sample a t.abstract
+end
+
+
+
+
 type slot = { mutable status : status; 
 	       mutable notify : notify;
 	       mutable active : bool }
-and status = Empty | NEmpty | Maybe
-and notify = Nothing | Do of slot * (slot -> unit) * notify
+and status = Empty | NEmpty of Witness.witness | Maybe
+and notify = Nothing | Do of slot * (Witness.witness -> unit) * notify
 
 let slot_empty = { status = Empty; active = false; notify = Nothing }
-let slot_not_empty = { status = NEmpty; active = false; notify = Nothing }
+let slot_nempty w = { status = NEmpty w;
+		     active = false; notify = Nothing }
 
-let rec notify = function
+let rec notify w = function
   | Nothing -> ()
   | Do (n,f,rem) -> 
-      if n.status == Maybe then (try f n with NotEmpty -> ());
-      notify rem
+      if n.status == Maybe then (try f w with NotEmpty -> ());
+      notify w rem
 
 let rec iter_s s f = function
   | [] -> ()
   | arg::rem -> f arg s; iter_s s f rem
 
 
-let set s =
-  s.status <- NEmpty;
-  notify s.notify;
+let set s w =
+  s.status <- NEmpty w;
+  notify w s.notify;
   s.notify <- Nothing; 
   raise NotEmpty
 
-let rec big_conj f l n =
+let rec big_conj f l n w =
   match l with
-    | [] -> set n
-    | [arg] -> f arg n
+    | [] -> set n w
+    | [arg] -> f w arg n
     | arg::rem ->
 	let s = 
 	  { status = Maybe; active = false; 
-	    notify = Do (n,(big_conj f rem), Nothing) } in
+	    notify = Do (n,(big_conj f rem n), Nothing) } in
 	try 
-	  f arg s;
+	  f w arg s;
 	  if s.active then n.active <- true
-	with NotEmpty -> if n.status == NEmpty then raise NotEmpty
-
-let guard a f n =
-  match a with
-    | { status = Empty } -> ()
-    | { status = Maybe } as s -> 
-	n.active <- true; 
-	s.notify <- Do (n,f,s.notify)
-    | { status = NEmpty } -> f n
-
-
-(* Fast approximation *)
-
-module ClearlyEmpty = 
-struct
-
-let memo = DescrHash.create 8191
-let marks = ref [] 
-
-let rec slot d =
-  if not ((Intervals.is_empty d.ints) && 
-	  (Atoms.is_empty d.atoms) &&
-	  (Chars.is_empty d.chars) &&
-	  (Abstract.is_empty d.abstract) &&
-	  (not d.absent)) then slot_not_empty 
-  else try DescrHash.find memo d
-  with Not_found ->
-    let s = { status = Maybe; active = false; notify = Nothing } in
-    DescrHash.add memo d s;
-    (try
-       iter_s s check_times (BoolPair.get d.times);  
-       iter_s s check_xml (BoolPair.get d.xml); 
-       iter_s s check_arrow (BoolPair.get d.arrow);
-       iter_s s check_record (get_record d.record);
-       if s.active then marks := s :: !marks else s.status <- Empty;
-     with
-	 NotEmpty -> ());
-    s
-
-and check_times (left,right) s =
-  let (accu1,accu2) = cap_product any any left in
-  let single_right (t1,t2) s =
-    let t1 = descr t1 and t2 = descr t2 in
-    if trivially_disjoint accu1 t1 || trivially_disjoint accu2 t2 then set s 
-    else
-      let accu1 = diff accu1 t1 in guard (slot accu1) set s;
-      let accu2 = diff accu2 t2 in guard (slot accu2) set s in
-  guard (slot accu1) (guard (slot accu2) (big_conj single_right right)) s
-
-and check_xml (left,right) s =
-  let (accu1,accu2) = cap_product any any_pair left in
-  let single_right (t1,t2) s =
-    let t1 = descr t1 and t2 = descr t2 in
-    if trivially_disjoint accu1 t1 || trivially_disjoint accu2 t2 then set s 
-    else
-      let accu1 = diff accu1 t1 in guard (slot accu1) set s;
-      let accu2 = diff accu2 t2 in guard (slot accu2) set s in
-  guard (slot accu1) (guard (slot accu2) (big_conj single_right right)) s
-
-and check_arrow (left,right) s =
-  let single_right (s1,s2) s =
-    let accu1 = descr s1 and accu2 = neg (descr s2) in
-    let single_left (t1,t2) s =
-      let accu1 = diff_t accu1 t1 in guard (slot accu1) set s;
-      let accu2 = cap_t  accu2 t2 in guard (slot accu2) set s
-    in
-    guard (slot accu1) (big_conj single_left left) s
-  in
-  big_conj single_right right s
-
-and check_record (labels,(oleft,left),rights) s =
-  let rec single_right (oright,right) s = 
-    let next =
-      (oleft && (not oright)) ||
-      exists (Array.length left)
-	(fun i -> trivially_disjoint left.(i) right.(i))
-    in
-    if next then set s
-    else
-      for i = 0 to Array.length left - 1 do
-	let di = diff left.(i) right.(i) in guard (slot di) set s
-      done
-  in
-  let rec start i s =
-    if (i < 0) then big_conj single_right rights s
-    else guard (slot left.(i)) (start (i - 1)) s
-  in
-  start (Array.length left - 1) s
-
-
-let is_empty d =
-  let s = slot d in
-  List.iter 
-    (fun s' -> 
-       if s'.status == Maybe then s'.status <- Empty; s'.notify <- Nothing) 
-    !marks;
-  marks := [];
-  s.status == Empty
-end
-
-let clearly_disjoint t1 t2 =
-(*
-  if trivially_disjoint t1 t2 then true
-  else
-    if ClearlyEmpty.is_empty (cap t1 t2) then
-      (Printf.eprintf "!\n"; true) else false
-*)
-  trivially_disjoint t1 t2 || ClearlyEmpty.is_empty (cap t1 t2) 
-
-(* TODO: need to invesigate when ClearEmpty is a good thing... *)
+	with NotEmpty when n.status == Empty || n.status == Maybe -> ()
 
 let memo = DescrHash.create 8191
 let marks = ref [] 
@@ -644,11 +761,15 @@ let complex = ref 0
 let rec slot d =
   incr complex;
   Stats.Counter.incr count_subtype; 
-  if not ((Intervals.is_empty d.ints) && 
-	  (Atoms.is_empty d.atoms) &&
-	  (Chars.is_empty d.chars) &&
-	  (Abstract.is_empty d.abstract) &&
-	  (not d.absent)) then slot_not_empty 
+  if d.absent then slot_nempty Witness.WAbsent
+  else if not (Intervals.is_empty d.ints) 
+  then slot_nempty (Witness.WInt (Intervals.sample d.ints))
+  else if not (Atoms.is_empty d.atoms) 
+  then slot_nempty (Witness.WAtom (Atoms.sample d.atoms))
+  else if not (Chars.is_empty d.chars) 
+  then slot_nempty (Witness.WChar (Chars.sample d.chars))
+  else if not (Abstract.is_empty d.abstract) 
+  then slot_nempty (Witness.WAbstract (Abstract.sample d.abstract))
   else try DescrHash.find memo d
   with Not_found ->
     let s = { status = Maybe; active = false; notify = Nothing } in
@@ -659,88 +780,97 @@ let rec slot d =
        iter_s s check_arrow (BoolPair.get d.arrow);
        iter_s s check_record (get_record d.record);
        if s.active then marks := s :: !marks else s.status <- Empty;
-     with
-	 NotEmpty -> ());
+     with NotEmpty -> ());
     s
 
-and check_times (left,right) s =
-  let rec aux accu1 accu2 right s = match right with
-    | (n1,n2)::right ->
-	let t1 = descr n1 and t2 = descr n2 in
-	if trivially_disjoint accu1 t1 || 
-	   trivially_disjoint accu2 t2 then (
-	     aux accu1 accu2 right s )
-	else (
-          let accu1' = diff accu1 t1 in 
-	  guard (slot accu1') (aux accu1' accu2 right) s;
+and guard n t f = match (slot t) with
+  | { status = Empty } -> ()
+  | { status = Maybe } as s -> n.active <- true; s.notify <- Do (n,f,s.notify)
+  | { status = NEmpty v } -> f v
 
-          let accu2' = diff accu2 t2 in 
-	  guard (slot accu2') (aux accu1 accu2' right) s  
-	)
-    | [] -> set s
+and check_times (left,right) s =
+  let rec aux w1 w2 accu1 accu2 seen = function
+    (* Find a product in right which contains (w1,w2) *)
+    | [] -> (* no such product: the current witness is in the difference. *)
+	set s (Witness.wpair w1 w2)
+    | (n1,n2) :: rest 
+	when (Witness.node_has n1 w1) && (Witness.node_has n2 w2) ->
+	let right = seen @ rest in
+	let accu2' = diff accu2 (descr n2) in 
+	guard s accu2' (fun w2 -> aux w1 w2 accu1 accu2' [] right);
+	let accu1' = diff accu1 (descr n1) in
+	guard s accu1' (fun w1 -> aux w1 w2 accu1' accu2 [] right)
+    | k :: rest -> aux w1 w2 accu1 accu2 (k::seen) rest
   in
-  let (accu1,accu2) = cap_product any any left in
-  guard (slot accu1) (guard (slot accu2) (aux accu1 accu2 right)) s
+  let (t1,t2) = cap_product any any left in
+  guard s t1 (fun w1 -> guard s t2 (fun w2 -> aux w1 w2 t1 t2 [] right))
 
 and check_xml (left,right) s =
-  let rec aux accu1 accu2 right s = match right with
-    | (n1,n2)::right ->
-	let t1 = descr n1 and t2 = descr n2 in
-	if clearly_disjoint accu1 t1 || 
-	   trivially_disjoint accu2 t2 then (
-	     aux accu1 accu2 right s )
-	else (
-          let accu1' = diff accu1 t1 in 
-	  guard (slot accu1') (aux accu1' accu2 right) s;
-
-          let accu2' = diff accu2 t2 in 
-	  guard (slot accu2') (aux accu1 accu2' right) s  
-	)
-    | [] -> set s
+  let rec aux w1 w2 accu1 accu2 seen = function
+    (* Find a product in right which contains (w1,w2) *)
+    | [] -> (* no such product: the current witness is in the difference. *)
+	set s (Witness.wxml w1 w2)
+    | (n1,n2) :: rest 
+	when (Witness.node_has n1 w1) && (Witness.node_has n2 w2) ->
+	let right = seen @ rest in
+	let accu2' = diff accu2 (descr n2) in 
+	guard s accu2' (fun w2 -> aux w1 w2 accu1 accu2' [] right);
+	let accu1' = diff accu1 (descr n1) in
+	guard s accu1' (fun w1 -> aux w1 w2 accu1' accu2 [] right)
+    | k :: rest -> aux w1 w2 accu1 accu2 (k::seen) rest
   in
-  let (accu1,accu2) = cap_product any any_pair left in
-  guard (slot accu1) (guard (slot accu2) (aux accu1 accu2 right)) s
+  let (t1,t2) = cap_product any any_pair left in
+  guard s t1 (fun w1 -> guard s t2 (fun w2 -> aux w1 w2 t1 t2 [] right))
 
 and check_arrow (left,right) s =
-  let single_right (s1,s2) s =
-    let rec aux accu1 accu2 left s = match left with
+  let single_right f (s1,s2) s =
+    let rec aux w1 w2 accu1 accu2 left = match left with
       | (t1,t2)::left ->
           let accu1' = diff_t accu1 t1 in 
-	  guard (slot accu1') (aux accu1' accu2 left) s;
+	  guard s accu1' (fun w1 -> aux w1 w2 accu1' accu2 left);
 
           let accu2' = cap_t  accu2 t2 in 
-	  guard (slot accu2') (aux accu1 accu2' left) s
-      | [] -> set s
+	  guard s accu2' (fun w2 -> aux w1 (Some w2) accu1 accu2' left)
+      | [] -> 
+	  let f = match f with Witness.WFun (f,_) -> f | _ -> assert false in
+	  set s (Witness.wfun ((w1,w2)::f))
     in
     let accu1 = descr s1 in
-    guard (slot accu1) (aux accu1 (neg (descr s2)) left) s
+    guard s accu1 (fun w1 -> aux w1 None accu1 (neg (descr s2)) left)
   in
-  big_conj single_right right s
+  big_conj single_right right s (Witness.wfun [])
 
 and check_record (labels,(oleft,left),rights) s =
-  let rec aux left rights s = match rights with
-    | [] -> set s
-    | (oright,right)::rights ->
-	let next =
-	  (oleft && (not oright)) ||
-	  exists (Array.length left)
-	    (fun i -> trivially_disjoint left.(i) right.(i))
-	in
-	if next then aux left rights s
-	else
-	  for i = 0 to Array.length left - 1 do
-	    let left' = Array.copy left in
-	    let di = diff left.(i) right.(i) in
-	    left'.(i) <- di;
-	    guard (slot di) (aux left' rights) s;
-	  done
+  let rec aux ws accus seen = function
+    | [] ->
+	let rec aux w i = function
+	  | [] -> assert (i == Array.length ws); w
+	  | l::labs -> 
+	      let w = match ws.(i) with 
+		| Witness.WAbsent -> w
+		| wl -> LabelMap.add l wl w in
+	      aux w (succ i) labs in
+	set s (Witness.wrecord (aux LabelMap.empty 0 labels) oleft)
+    | (false,_) :: rest when oleft -> aux ws accus seen rest
+    | (_,f) :: rest 
+	when not (exists (Array.length left)
+		    (fun i -> not (Witness.type_has f.(i) ws.(i)))) ->
+	(* TODO: a version f get_record which keeps nodes in neg records. *)
+	let right = seen @ rest in
+	for i = 0 to Array.length left - 1 do
+	  let di = diff accus.(i) f.(i) in
+	  guard s di (fun wi -> 
+			let accus' = Array.copy accus in accus'.(i) <- di;
+			let ws' = Array.copy ws in ws'.(i) <- wi;
+			aux ws' accus' [] right);
+	done
+    | k :: rest -> aux ws accus (k::seen) rest
   in
-  let rec start i s =
-    if (i < 0) then aux left rights s
-    else
-      guard (slot left.(i)) (start (i - 1)) s
+  let rec start wl i =
+    if (i < 0) then aux (Array.of_list wl) left [] rights
+    else guard s left.(i) (fun w -> start (w::wl) (i - 1))
   in
-  start (Array.length left - 1) s
+  start [] (Array.length left - 1)
 
 
 
@@ -748,16 +878,21 @@ let timer_subtype = Stats.Timer.create "Types.is_empty"
 
 
 let is_empty d =
-  Stats.Timer.start timer_subtype; 
+  Stats.Timer.start timer_subtype;
   let s = slot d in
   List.iter 
     (fun s' -> 
        if s'.status == Maybe then s'.status <- Empty; s'.notify <- Nothing) 
     !marks;
   marks := [];
-  Stats.Timer.stop timer_subtype 
+  Stats.Timer.stop timer_subtype
     (s.status == Empty)
 
+let getwit t = match (slot t).status with NEmpty w -> w | _ -> assert false
+  (* Assumes that is_empty has been called on t before. *)
+
+let witness t = if is_empty t then raise Not_found else getwit t
+
 (*
 let is_empty d =
 (*  let b1 = ClearlyEmpty.is_empty d in
@@ -785,14 +920,90 @@ let is_empty d =
 let non_empty d = 
   not (is_empty d)
 
-let subtype d1 d2 =
-  is_empty (diff d1 d2)
-
 let disjoint d1 d2 =
-  is_empty (cap d1 d2)
+(*  try match (DescrHash.find memo d1).status with
+    | Empty -> true
+    | NEmpty w when Witness.type_has d2 w -> false
+    | _ -> raise Not_found
+  with Not_found ->
+  try match (DescrHash.find memo d2).status with
+    | Empty -> true
+    | NEmpty w when Witness.type_has d1 w -> false
+    | _ -> raise Not_found
+  with Not_found -> *)
+    is_empty (cap d1 d2)
+
+let subtype d1 d2 =
+(*  try match (DescrHash.find memo d1).status with
+    | Empty -> true
+    | NEmpty w when not (Witness.type_has d2 w) -> false
+    | _ -> raise Not_found
+  with Not_found -> *)
+    is_empty (diff d1 d2)
 
 let equiv d1 d2 = (subtype d1 d2) && (subtype d2 d1)
 
+
+module Cache = struct
+(*
+  let type_has_witness t w =
+    Format.fprintf Format.std_formatter
+      "check wit:%a@." print_witness w; 
+    let r = type_has_witness t w in
+    Format.fprintf Format.std_formatter "Done@."; 
+    r
+*)
+
+  type 'a cache =
+    | Empty
+    | Type of t * 'a
+    | Split of Witness.witness * 'a cache * 'a cache
+
+  let rec find f t = function
+    | Empty -> 
+	let r = f t in Type (t,r), r
+    | Split (w,yes,no) ->
+	if Witness.type_has t w 
+	then let yes,r = find f t yes in Split (w,yes,no), r
+	else let no,r = find f t no in Split (w,yes,no), r
+    | Type (s,rs) as c ->
+	let f1 ()= 
+	  let w = witness (diff t s) in 
+	  let rt = f t in 
+	  Split (w, Type (t,rt), c), rt
+	and f2 () =
+	  let w = witness (diff s t) in 
+	  let rt = f t in
+	  Split (w, c, Type (t,rt)), rt in
+
+	if Random.int 2 = 0 then
+	  try f1 () with Not_found -> try f2 () with Not_found -> c, rs
+	else
+	  try f2 () with Not_found -> try f1 () with Not_found -> c, rs
+
+  let rec lookup t = function
+    | Empty -> None
+    | Split (w,yes,no) -> lookup t (if Witness.type_has t w then yes else no)
+    | Type (s,rs) -> if equiv s t then Some rs else None
+
+  let emp = Empty
+
+
+  let rec dump_cache f ppf = function
+    | Empty -> Format.fprintf ppf "Empty"
+    | Type (_,s) -> Format.fprintf ppf "*%a" f s
+    | Split (w,c1,c2) -> Format.fprintf ppf "?(%a,%a)"
+	(*Witness.print_witness w *)(dump_cache f) c1 (dump_cache f) c2
+
+  let memo f =
+    let c = ref emp in
+    fun t ->
+       let c',r = find f t !c in
+       c := c';
+       r
+
+end
+
 module Product =
 struct
   type t = (descr * descr) list
@@ -878,7 +1089,7 @@ struct
   let pi2 = List.fold_left (fun acc (_,t2) -> cup acc t2) empty
   let pi2_restricted restr = 
     List.fold_left (fun acc (t1,t2) -> 
-		      if is_empty (cap t1 restr) then acc
+		      if disjoint t1 restr then acc
 		      else cup acc t2) empty
 
   let restrict_1 rects pi1 =
@@ -933,12 +1144,27 @@ struct
 
   let constraint_on_2 n t1 =
     List.fold_left 
-      (fun accu (d1,d2) ->
-	 if is_empty (cap d1 t1) then accu else cap accu d2)
+      (fun accu (d1,d2) -> if disjoint d1 t1 then accu else cap accu d2)
       any
       n
 
 
+  let merge_same_first tr =
+    let trs = ref [] in
+    let _ = 
+      List.fold_left
+	(fun memo (t1,t2) ->
+	   let memo',l = 
+	     Cache.find 
+	       (fun t1 -> let l = ref empty in trs := (t1,l) :: !trs; l) 
+	       t1 memo in
+	   l := cup t2 !l;
+	   memo')
+	Cache.emp tr in
+    List.map (fun (t1,l) -> (t1,!l)) !trs
+    
+
+  (* same on second component: use the same implem? *)
   let clean_normal l =
     let rec aux accu (t1,t2) =
       match accu with
@@ -1029,7 +1255,6 @@ struct
   let split_normal d l =
     TR.boolean_normal (aux_split d l)
 
-
   let pi l d = TR.pi1 (split d l)
 
   let project d l =
@@ -1137,6 +1362,16 @@ struct
 	  (split d l)
     in
     aux [] [] d
+
+  type t = TR.t
+  let focus = split_normal
+  let get_this r = { (TR.pi1 r) with absent = false }
+  let need_others = function _::_::_ -> true | _ -> false
+  let constraint_on_others r t1 =
+    List.fold_left 
+      (fun accu (d1,d2) -> if disjoint d1 t1 then accu else cap accu d2)
+      any_record
+      r
 end
 
 
@@ -1201,9 +1436,18 @@ struct
 
   module DescrPairMap = Map.Make(Custom.Pair(Descr)(Descr))
 
+  let memo = ref Cache.emp
+
+  let uniq t = 
+    let c',r = Cache.find (fun t -> t) t !memo in
+    memo := c';
+    r
+  let lookup t = match Cache.lookup t !memo with Some t -> t | None -> t
+
   let named = ref DescrMap.empty
   let named_xml = ref DescrPairMap.empty
   let register_global cu (name : Ns.QName.t) d = 
+    let d = uniq d in
     if equal { d with xml = BoolPair.empty } empty then 
       (let l = (*Product.merge_same_2*) (Product.get ~kind:`XML d) in
        match l with
@@ -1216,6 +1460,7 @@ struct
     else named := DescrMap.add d (cu,name) !named
 	
   let unregister_global d =
+    let d = uniq d in
     if equal { d with xml = BoolPair.empty } empty then
       (let l = Product.get ~kind:`XML d in
       match l with
@@ -1260,6 +1505,7 @@ struct
     n >= 5
 
   let rec prepare d =
+    let d = lookup d in
     try DescrHash.find memo d
     with Not_found ->
       try 
@@ -1330,10 +1576,29 @@ struct
     Decompile.decompile 
       (fun t -> 
 	 let tr = Product.get t in 
+	 let tr = Product.merge_same_first tr in
 	 let tr = Product.clean_normal tr in
 
-	 let tr = List.map (fun (l,t) -> prepare l, t) tr in
-	 tr, Atoms.contains nil_atom t.atoms)
+	 let eps = Atoms.contains nil_atom t.atoms in
+	 let tr_cons = List.map (fun (li,ti) -> (cons li, cons ti)) tr in
+
+	 try
+	   let (l0,t0) = List.find
+	     (fun ((l0,t0) as tr0) ->
+	       let t'' = 
+		 List.fold_left
+		   (fun accu ((li,ti) as tri) ->
+		     if tr0 == tri then accu
+		     else cup accu (times li ti)
+		   )
+		   (if eps then nil_type else empty) 
+		   tr_cons
+	       in
+	       equiv (descr t0) t'' ) tr_cons in
+	   `Eps (prepare (descr l0), descr t0)
+	 with Not_found -> 
+	   let tr = List.map (fun (l,t) -> prepare l, t) tr in
+	   `T (tr, eps))
       d
 
   let gen = ref 0
@@ -1491,6 +1756,7 @@ struct
     | _ -> assert false
 
   let print ppf t =
+    let t = uniq t in
     let t = prepare t in
     assign_name t;
     Format.fprintf ppf "@[@[%a@]" (do_print_slot 0) t;
