@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 open Ident
@@ -195,6 +195,13 @@ module Record : sig
   val remove_field: t -> label -> t
 
   val get: t -> ((bool * t) label_map * bool * bool) list
+
+
+  type t
+  val focus: descr -> label -> t
+  val get_this: t -> descr
+  val need_others: t -> bool
+  val constraint_on_others: t -> descr -> descr
 end
 
 module Arrow : sig
@@ -282,4 +289,17 @@ sig
   val to_string: t -> string
 end
 
+module Witness: sig
+  type witness
+  val print_witness: Format.formatter -> witness -> unit
+end
+val witness: t -> Witness.witness
 
+
+module Cache: sig
+  type 'a cache
+  val emp: 'a cache
+  val find: (t -> 'a) -> t -> 'a cache -> 'a cache * 'a
+
+  val memo: (t -> 'a) -> (t -> 'a)
+end
