@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 open Ident
@@ -540,6 +540,15 @@ let deferr s = raise (Patterns.Error s)
   let mk_weakstar n = WeakStar n
   let mk_seqcapt x n = SeqCapture (x,n)
 
+  let mk_str s =
+    let j = Encodings.Utf8.end_index s in
+    let rec aux i =
+      if Encodings.Utf8.equal_index i j then Epsilon
+      else let (c,i) = Encodings.Utf8.next s i in
+      let t = Types.char (Chars.atom (Chars.V.mk_int c)) in
+      Seq(Elem(mk_type t), aux i) in
+    aux (Encodings.Utf8.start_index s)
+
   let rec prepare_regexp vars b rvars f = function
       (* - vars: seq variables to be propagated top-down and added
 	 to each captured element
@@ -704,7 +713,7 @@ let deferr s = raise (Patterns.Error s)
 	    (fun accu (t1,t2) -> mk_or accu (mk_prod (mk_type t1) (aux t2)))
 	    (if Types.Atom.has_atom t Sequence.nil_atom then q else iempty)
 	    (Types.Product.get t) in
-	n.desc <- d.desc;
+	n.desc <- ILink d;
 	n
     in
     aux t
