@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005,2006 (see LICENSE for details)   *)
 (**************************************************************************)
 
 open Location
@@ -599,11 +599,13 @@ and if_then_else loc cond yes no =
     Typed.br_branches = [
       { Typed.br_loc = yes.Typed.exp_loc;
 	Typed.br_used = false;
+	Typed.br_ghost = false;
 	Typed.br_vars_empty = Fv.empty;
 	Typed.br_pat = pat_true;
 	Typed.br_body = yes };
       { Typed.br_loc = no.Typed.exp_loc;
 	Typed.br_used = false;
+	Typed.br_ghost = false;
 	Typed.br_vars_empty = Fv.empty;
 	Typed.br_pat = pat_false;
 	Typed.br_body = no } ];
@@ -722,10 +724,12 @@ and branches env b =
     let fv2 = Fv.diff fv2 fvp in
     fv := Fv.cup !fv fv2;
     accept := Types.cup !accept (Types.descr (Patterns.accept p));
+    let ghost = br_loc == noloc in
     let br = 
       { 
 	Typed.br_loc = br_loc;
-	Typed.br_used = br_loc == noloc;
+	Typed.br_used = ghost;
+	Typed.br_ghost = ghost;
 	Typed.br_vars_empty = fvp;
 	Typed.br_pat = p;
 	Typed.br_body = e } in
@@ -767,6 +771,7 @@ and select_from_where env loc e from where =
     (* transform e with p -> ... *)
     let br = { Typed.br_loc = ploc;
 	  Typed.br_used = false;
+	  Typed.br_ghost = false;
 	  Typed.br_vars_empty = fvp;
 	  Typed.br_pat = p;
 	  Typed.br_body = rest } in
@@ -845,6 +850,7 @@ let flatten arg constr precise =
     let t = arg sconstr' true in
     verify_noloc (Sequence.flatten t) constr
 
+
 let rec type_check env e constr precise = 
   let d = type_check' e.exp_loc env e.exp_descr constr precise in
   let d = if precise then d else constr in
@@ -1033,31 +1039,34 @@ and type_check_string loc env ofs s i j e constr precise =
       else constr
 
 and type_record loc env r constr precise =
-(* try to get rid of precise = true for values of fields *)
-(* also: the use equivalent of need_second to optimize... *)
   if not (Types.Record.has_record constr) then
     should_have loc constr "but it is a record";
   let (rconstr,res) = 
     List.fold_left
       (fun (rconstr,res) (l,e) ->
-	 (* could compute (split l e) once... *)
-	 let pi = Types.Record.project_opt rconstr l in
+	 let r = Types.Record.focus rconstr l in
+	 let pi = Types.Record.get_this r in
 	 if Types.is_empty pi then 
 	   (let l = Label.string_of_attr l in
 	    should_have loc constr
 	      (Printf.sprintf "Field %s is not allowed here." l));
-	 let t = type_check env e pi true in
-	 let rconstr = Types.Record.condition rconstr l t in
-	 let res = (l,Types.cons t) :: res in
+	 let t = type_check env e pi (precise || Types.Record.need_others r) in
+	 let rconstr = Types.Record.constraint_on_others r t in
+	 if Types.is_empty rconstr then
+	   (let l = Label.string_of_attr l in
+	    should_have loc constr
+	      (Printf.sprintf "Type of field %s is not precise enough." l));
+
+	 let res = if precise then LabelMap.add l (Types.cons t) res 
+	 else res in
 	 (rconstr,res)
-      ) (constr, []) (LabelMap.get r)
+      ) (constr, LabelMap.empty) (LabelMap.get r)
   in
   if not (Types.Record.has_empty_record rconstr) then
     should_have loc constr "More fields should be present";
-  let t = 
-    Types.record_fields (false, LabelMap.from_list (fun _ _ -> assert false) res)
-  in
-  verify loc t constr
+  if precise then Types.record_fields (false, res)
+  else constr
+
 
 
 and type_check_branches loc env targ brs constr precise =
@@ -1144,8 +1153,8 @@ and type_rec_funs env l =
 let rec unused_branches b =
   List.iter
     (fun (Branch (br,s)) -> 
-       if not br.br_used 
-       then warning br.br_loc "This branch is not used"
+       if br.br_ghost then ()
+       else if not br.br_used then warning br.br_loc "This branch is not used"
        else (
 	 if not (IdSet.is_empty br.br_vars_empty)
 	 then (
